#![no_std]
#![feature(alloc_error_handler)]
#![feature(int_roundings)]
#![feature(lang_items)]
#![allow(internal_features)]
#![feature(let_chains)]

extern crate alloc;

use self::os::{OsMemoryEntry, OsMemoryKind};

#[macro_use]
mod os;
mod logger;
mod serial;

//TODO: allocate this in a more reasonable manner
static mut AREAS: [OsMemoryEntry; 1024] = [OsMemoryEntry {
	base: 0,
	size: 0,
	kind: OsMemoryKind::Null,
}; 1024];
static mut AREAS_LEN: usize = 0;

pub fn area_add(area: OsMemoryEntry) {
	unsafe {
		for existing_area in &mut AREAS[0..AREAS_LEN] {
			if existing_area.kind == area.kind {
				if existing_area.base.unchecked_add(existing_area.size) == area.base {
					existing_area.size += area.size;
					return;
				}
				if area.base.unchecked_add(area.size) == existing_area.base {
					existing_area.base = area.base;
					return;
				}
			}
		}
		*AREAS.get_mut(AREAS_LEN).expect("AREAS overflowed!") = area;
		AREAS_LEN += 1;
	}
}

pub static mut LIVE_OPT: Option<(u64, &'static [u8])> = None;

#[allow(dead_code)]
#[derive(Debug)]
#[repr(C, packed(8))]
pub struct KernelArgs {
	kernel_base: u64,
	kernel_size: u64,
	stack_base: u64,
	stack_size: u64,
	env_base: u64,
	env_size: u64,

	/// The base pointer to the saved RSDP.
	///
	/// This field can be NULL, and if so, the system has not booted with UEFI or in some other way
	/// retrieved the RSDPs. The kernel or a userspace driver will thus try searching the BIOS
	/// memory instead. On UEFI systems, searching is not guaranteed to actually work though.
	acpi_rsdp_base: u64,
	/// The size of the RSDP region.
	acpi_rsdp_size: u64,

	areas_base: u64,
	areas_size: u64,

	bootstrap_base: u64,
	bootstrap_size: u64,
}

fn main() {}
